
/*
	link csi.c (csi.o) compiled with -DCSI_RGB
*/

#include <stdio.h>
#include <stdlib.h>
#include "cell.h"
#include "rif_rgb.h"
#include "csi.h"
#include "sif.h"
#include "sif_rgb.h"

extern void	Error();

#define PATH_LEN	1024
#define ALLOC(type,noe)	(type *)malloc(sizeof(type)*(size_t)(noe))
#define CELL(data)	(Cell)(data+0.5)

int	main(argc,argv)
int	argc;
char	**argv;
{
	char	**oPath,**mPath,format[PATH_LEN],nPath[PATH_LEN],
		*mae="memory allocatoin error.";
	int	Nx,Ny,Nz,oBPS,x,y,z,mBPS,len,ox,oy,mx,my,gray;
	Cell	**R,**G,**B,**oR,**oG,**oB,**mR,**mG,**mB,r,g,b;
	double	ratio,factor,ratio0,ratio1;

	if (argc!=7)
	    Error("usage : si_cim orgDir orgName mulDir mulName ratio newDir");

	oPath=CheckSliceImages(argv[1],argv[2],&Nx,&Ny,&Nz,&oBPS);
	mPath=CheckSliceImages(argv[3],argv[4],&x, &y, &z, &mBPS);
	if (Nx!=x || Ny!=y || Nz!=z) Error("image size not match.");

	if ((R=ALLOC(Cell *,Ny))==NULL ||
	    (G=ALLOC(Cell *,Ny))==NULL ||
	    (B=ALLOC(Cell *,Ny))==NULL) Error(mae);

	for (y=0; y<Ny; y++)
	    if ((R[y]=ALLOC(Cell,Nx))==NULL ||
		(G[y]=ALLOC(Cell,Nx))==NULL ||
		(B[y]=ALLOC(Cell,Nx))==NULL) Error(mae);

	if ((ratio=atof(argv[5]))<0.0 || ratio>1.0) Error("bad ratio.");

	factor=255.0/(double)((1<<oBPS)-1);
	ratio0=factor*ratio;
	ratio1=factor*(1.0-ratio)/(double)((1<<mBPS)-1);

	len=1; for (z=10; z<Nz; z*=10) ++len;
	(void)sprintf(format,"%s/%%0%dd.tif",argv[6],len);

	for (z=0; z<Nz; z++) {
	    ReadImageFile_RGB(oPath[z],&ox,&oy,NULL,&oR,&oG,&oB,NULL);
	    ReadImageFile_RGB(mPath[z],&mx,&my,NULL,&mR,&mG,&mB,NULL);

	    gray=1;
	    for (y=0; y<Ny; y++)
	    for (x=0; x<Nx; x++) {
		if (x>=ox || y>=oy)
		    r=g=b=0;
		else if (x>=mx || y>=my) {
		    r=CELL((double)oR[y][x]*ratio0);
		    g=CELL((double)oG[y][x]*ratio0);
		    b=CELL((double)oB[y][x]*ratio0);
		}
		else {
		    r=CELL((double)oR[y][x]*(ratio0+ratio1*(double)mR[y][x]));
		    g=CELL((double)oG[y][x]*(ratio0+ratio1*(double)mG[y][x]));
		    b=CELL((double)oB[y][x]*(ratio0+ratio1*(double)mB[y][x]));
		}
		R[y][x]=r;
		G[y][x]=g;
		B[y][x]=b; if (gray) gray=r==g && g==b;
	    }
	    (void)sprintf(nPath,format,z);
	    if (gray)
		StoreImageFile(nPath,Nx,Ny,8,R,NULL);
	    else
		StoreImageFile_RGB(nPath,Nx,Ny,8,R,G,B,NULL);

	    for (y=my-1; y>=0; y--) {
		free(mB[y]); free(mG[y]); free(mR[y]);
	    }
	    free(mB); free(mG); free(mR);

	    for (y=oy-1; y>=0; y--) {
		free(oB[y]); free(oG[y]); free(oR[y]);
	    }
	    free(oB); free(oG); free(oR);
	}
	return 0;
}
